From: Wen Gu <guwen@linux.alibaba.com>
To: kgraul@linux.ibm.com, wenjia@linux.ibm.com, jaka@linux.ibm.com,
davem@davemloft.net, edumazet@google.com, kuba@kernel.org,
pabeni@redhat.com
Cc: alibuda@linux.alibaba.com, tonylu@linux.alibaba.com,
guwen@linux.alibaba.com, linux-s390@vger.kernel.org,
netdev@vger.kernel.org, linux-kernel@vger.kernel.org
Subject: [PATCH net-next 17/18] net/smc: modify cursor update logic when sndbuf mapped to RMB
Date: Tue, 19 Sep 2023 22:42:01 +0800 [thread overview]
Message-ID: <1695134522-126655-18-git-send-email-guwen@linux.alibaba.com> (raw)
In-Reply-To: <1695134522-126655-1-git-send-email-guwen@linux.alibaba.com>
Since local sndbuf shares the same physical memory with peer RMB after
mapping, the logic related to cursor update needs to be adapted to ensure
that the data written by local won't overwrite the data that has not been
consumed by the peer.
So in this case, the fin_curs and sndbuf_space that were originally
updated after sending the CDC message should be modified to not be update
until the peer updates cons_curs.
Signed-off-by: Wen Gu <guwen@linux.alibaba.com>
---
net/smc/smc_cdc.c | 50 +++++++++++++++++++++++++++++++++++++++-----------
1 file changed, 39 insertions(+), 11 deletions(-)
diff --git a/net/smc/smc_cdc.c b/net/smc/smc_cdc.c
index 2641800..7cc677d 100644
--- a/net/smc/smc_cdc.c
+++ b/net/smc/smc_cdc.c
@@ -18,6 +18,7 @@
#include "smc_tx.h"
#include "smc_rx.h"
#include "smc_close.h"
+#include "smc_ism.h"
/********************************** send *************************************/
@@ -256,17 +257,24 @@ int smcd_cdc_msg_send(struct smc_connection *conn)
return rc;
smc_curs_copy(&conn->rx_curs_confirmed, &curs, conn);
conn->local_rx_ctrl.prod_flags.cons_curs_upd_req = 0;
- /* Calculate transmitted data and increment free send buffer space */
- diff = smc_curs_diff(conn->sndbuf_desc->len, &conn->tx_curs_fin,
- &conn->tx_curs_sent);
- /* increased by confirmed number of bytes */
- smp_mb__before_atomic();
- atomic_add(diff, &conn->sndbuf_space);
- /* guarantee 0 <= sndbuf_space <= sndbuf_desc->len */
- smp_mb__after_atomic();
- smc_curs_copy(&conn->tx_curs_fin, &conn->tx_curs_sent, conn);
+ if (!smc_ism_dmb_mappable(conn->lgr->smcd)) {
+ /* If local sndbuf has been mapped to peer RMB, then
+ * don't update the tx_curs_fin and sndbuf_space until
+ * peer has consumed the data in RMB.
+ */
- smc_tx_sndbuf_nonfull(smc);
+ /* Calculate transmitted data and increment free send buffer space */
+ diff = smc_curs_diff(conn->sndbuf_desc->len, &conn->tx_curs_fin,
+ &conn->tx_curs_sent);
+ /* increased by confirmed number of bytes */
+ smp_mb__before_atomic();
+ atomic_add(diff, &conn->sndbuf_space);
+ /* guarantee 0 <= sndbuf_space <= sndbuf_desc->len */
+ smp_mb__after_atomic();
+ smc_curs_copy(&conn->tx_curs_fin, &conn->tx_curs_sent, conn);
+
+ smc_tx_sndbuf_nonfull(smc);
+ }
return rc;
}
@@ -324,7 +332,7 @@ static void smc_cdc_msg_recv_action(struct smc_sock *smc,
{
union smc_host_cursor cons_old, prod_old;
struct smc_connection *conn = &smc->conn;
- int diff_cons, diff_prod;
+ int diff_cons, diff_prod, diff_tx;
smc_curs_copy(&prod_old, &conn->local_rx_ctrl.prod, conn);
smc_curs_copy(&cons_old, &conn->local_rx_ctrl.cons, conn);
@@ -340,6 +348,26 @@ static void smc_cdc_msg_recv_action(struct smc_sock *smc,
atomic_add(diff_cons, &conn->peer_rmbe_space);
/* guarantee 0 <= peer_rmbe_space <= peer_rmbe_size */
smp_mb__after_atomic();
+
+ if (conn->lgr->is_smcd &&
+ smc_ism_dmb_mappable(conn->lgr->smcd)) {
+ /* If local sndbuf has been mapped to peer RMB, then
+ * update tx_curs_fin and sndbuf_space when peer has
+ * consumed the data in it's RMB.
+ */
+
+ /* calculate peer rmb consumed data */
+ diff_tx = smc_curs_diff(conn->sndbuf_desc->len, &conn->tx_curs_fin,
+ &conn->local_rx_ctrl.cons);
+ /* increase local sndbuf space and fin_curs */
+ smp_mb__before_atomic();
+ atomic_add(diff_tx, &conn->sndbuf_space);
+ /* guarantee 0 <= sndbuf_space <= sndbuf_desc->len */
+ smp_mb__after_atomic();
+ smc_curs_copy(&conn->tx_curs_fin, &conn->local_rx_ctrl.cons, conn);
+
+ smc_tx_sndbuf_nonfull(smc);
+ }
}
diff_prod = smc_curs_diff(conn->rmb_desc->len, &prod_old,
--
1.8.3.1
next prev parent reply other threads:[~2023-09-19 14:44 UTC|newest]
Thread overview: 29+ messages / expand[flat|nested] mbox.gz Atom feed top
2023-09-19 14:41 [PATCH net-next 00/18] net/smc: implement virtual ISM extension and loopback-ism Wen Gu
2023-09-19 14:41 ` [PATCH net-next 01/18] net/smc: decouple ism_dev from SMC-D device dump Wen Gu
2023-09-21 20:41 ` Simon Horman
2023-09-22 8:05 ` Wen Gu
2023-09-22 18:13 ` Gerd Bayer
2023-09-23 9:24 ` Wen Gu
2023-09-19 14:41 ` [PATCH net-next 02/18] net/smc: decouple ism_dev from SMC-D DMB registration Wen Gu
2023-09-19 14:41 ` [PATCH net-next 03/18] net/smc: extract v2 check helper from SMC-D device registration Wen Gu
2023-09-19 14:41 ` [PATCH net-next 04/18] net/smc: support SMCv2.x supplemental features negotiation Wen Gu
2023-09-19 14:41 ` [PATCH net-next 05/18] net/smc: reserve CHID range for SMC-D virtual device Wen Gu
2023-09-19 14:41 ` [PATCH net-next 06/18] net/smc: extend GID to 128bits for virtual ISM device Wen Gu
2023-09-20 7:02 ` kernel test robot
2023-09-20 8:11 ` Wen Gu
2023-09-20 9:00 ` Niklas Schnelle
2023-09-20 13:05 ` Wen Gu
2023-09-19 14:41 ` [PATCH net-next 07/18] net/smc: disable SEID on non-s390 architecture Wen Gu
2023-09-19 14:41 ` [PATCH net-next 08/18] net/smc: enable virtual ISM device feature bit Wen Gu
2023-09-19 14:41 ` [PATCH net-next 09/18] net/smc: introduce SMC-D loopback device Wen Gu
2023-09-19 14:41 ` [PATCH net-next 10/18] net/smc: implement ID-related operations of loopback Wen Gu
2023-09-19 14:41 ` [PATCH net-next 11/18] net/smc: implement some unsupported " Wen Gu
2023-09-19 14:41 ` [PATCH net-next 12/18] net/smc: implement DMB-related " Wen Gu
2023-09-23 15:24 ` kernel test robot
2023-09-24 8:54 ` Wen Gu
2023-09-19 14:41 ` [PATCH net-next 13/18] net/smc: register loopback device as SMC-Dv2 device Wen Gu
2023-09-19 14:41 ` [PATCH net-next 14/18] net/smc: add operation for getting DMB attribute Wen Gu
2023-09-19 14:41 ` [PATCH net-next 15/18] net/smc: add operations for DMB attach and detach Wen Gu
2023-09-19 14:42 ` [PATCH net-next 16/18] net/smc: avoid data copy from sndbuf to peer RMB in SMC-D Wen Gu
2023-09-19 14:42 ` Wen Gu [this message]
2023-09-19 14:42 ` [PATCH net-next 18/18] net/smc: add interface implementation of loopback device Wen Gu
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=1695134522-126655-18-git-send-email-guwen@linux.alibaba.com \
--to=guwen@linux.alibaba.com \
--cc=alibuda@linux.alibaba.com \
--cc=davem@davemloft.net \
--cc=edumazet@google.com \
--cc=jaka@linux.ibm.com \
--cc=kgraul@linux.ibm.com \
--cc=kuba@kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-s390@vger.kernel.org \
--cc=netdev@vger.kernel.org \
--cc=pabeni@redhat.com \
--cc=tonylu@linux.alibaba.com \
--cc=wenjia@linux.ibm.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox